package torrents

import (
	"encoding/json"
	"errors"
	"net/http"
	"strconv"

	"github.com/gorilla/mux"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"
)

// DownloadHandler downloads a movie via polochon
func DownloadHandler(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	var data struct {
		URL string `json:"url"`
	}
	err := json.NewDecoder(r.Body).Decode(&data)
	if err != nil {
		return env.RenderError(w, errors.New("failed to get the url"))
	}

	if data.URL == "" {
		return env.RenderError(w, errors.New("no given url"))
	}

	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return env.RenderError(w, errors.New("invalid user type"))
	}

	client, err := user.NewPapiClient()
	if err != nil {
		return env.RenderError(w, err)
	}

	err = client.AddTorrent(data.URL)
	if err != nil {
		return env.RenderError(w, err)
	}

	return env.RenderOK(w, "Torrent added")
}

// ListHandler downloads a torrent via polochon
func ListHandler(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return env.RenderError(w, errors.New("invalid user type"))
	}

	client, err := user.NewPapiClient()
	if err != nil {
		return env.RenderError(w, err)
	}

	torrents, err := client.GetTorrents()
	if err != nil {
		return env.RenderError(w, err)
	}

	return env.RenderJSON(w, torrents)
}

// RemoveHandler removes a torrents via polochon
func RemoveHandler(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return env.RenderError(w, errors.New("invalid user type"))
	}

	// Get the torrent ID from the URL
	vars := mux.Vars(r)
	torrentID, err := strconv.Atoi(vars["id"])
	if err != nil {
		return env.RenderError(w, errors.New("invalid argument"))
	}

	client, err := user.NewPapiClient()
	if err != nil {
		return env.RenderError(w, err)
	}

	err = client.RemoveTorrent(torrentID)
	if err != nil {
		return env.RenderError(w, err)
	}

	return env.RenderOK(w, "Torrent removed")
}
